/*
 * RSSBot
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.rssbot.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.commands.converters.impl.string
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.ephemeralSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import com.prof18.rssparser.RssParser
import com.prof18.rssparser.model.RssItem
import net.moonleay.rssbot.data.database.entry.RSSData
import net.moonleay.rssbot.data.database.entry.SubscriptionData
import net.moonleay.rssbot.data.database.repository.RSSRepository
import net.moonleay.rssbot.data.database.repository.SubscriptionRepository
import net.moonleay.rssbot.data.database.tables.SubscriptionsTable
import net.moonleay.rssbot.extensions.components.FeedColor
import net.moonleay.rssbot.util.EmbedColor
import net.moonleay.rssbot.util.Logger
import net.moonleay.rssbot.util.MessageUtil

class FeedExtension : Extension() {

    override val name = "feed"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        ephemeralSlashCommand(::FeedArguments) {
            name = "feed"
            description = "Feed this channel a new feed"
            this.action {
                val feedName = arguments.feedName
                val feedColor = arguments.feedColor
                val feedUrl = arguments.feedUrl
                val user = this.user.asUser()

                if (feedName == null || feedColor == null || feedUrl == null) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Missing Arguments",
                                "You are missing one or more arguments",
                                user.username
                            )
                        )
                    }
                    return@action
                }
                val parser = RssParser()
                var title = "Not found"
                var items = listOf<RssItem>()
                runCatching { // this sucks
                    val rss = parser.getRssChannel(feedUrl)
                    title = rss.title!!
                    items = rss.items
                }.onFailure {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Invalid Feed",
                                "The feed you provided is invalid",
                                user.username
                            )
                        )
                    }
                    return@action
                }
                if (SubscriptionRepository.exists(this.channel.id.value.toLong(), feedName) || SubscriptionRepository.existsWithUrl(this.channel.id.value.toLong(), feedUrl)) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "Feed already exists",
                                "The feed \"${feedName}\" already exists",
                                user.username
                            )
                        )
                    }
                    return@action
                }
                val id = SubscriptionRepository.write(
                    SubscriptionData(
                        0,
                        this.guild!!.id.value.toLong(),
                        this.channel.id.value.toLong(),
                        feedColor,
                        feedName,
                        feedUrl,
                        System.currentTimeMillis()
                    )
                )
                Logger.out("added feed as id ${id.resultedValues!!.first()[SubscriptionsTable.id]}")
                items.forEach {
                    RSSRepository.write(RSSData(it.guid!!, id.resultedValues!!.first()[SubscriptionsTable.id]))
                }
                this.respond {
                    this.embeds.add(
                        MessageUtil.getEmbed(
                            EmbedColor.INFO,
                            "Added Feed \"${feedName}\"",
                            "The feed \"${title}\" has been added to this channel",
                            user.username
                        )
                    )
                }
            }
        }
    }

    inner class FeedArguments : Arguments() {
        val feedName by string {
            this.name = "name"
            this.description = "The name of the feed"
        }

        val feedColor by enumChoice<FeedColor> {
            this.name = "color"
            this.description = "The color of the feed"
            this.typeName = "en_US"
        }

        val feedUrl by string {
            this.name = "url"
            this.description = "The feed url"
        }
    }
}
